@@ -58,9 +58,10 @@ class MidiFile():
 	"""This class parses a MIDI file."""
 	bpm = 120
 	meter = (4,4)
+	bytes_read = 0
 
 	def MIDI_to_Composition(self, file):
-		header, track_data, bytes = self.parse_midi_file(file)
+		header, track_data = self.parse_midi_file(file)
 
 		c = Composition()
 		if header[2]["fps"]:
@@ -76,7 +77,6 @@ class MidiFile():
 			meter = (4,4)
 			key = 'C'
 
-
 			for e in track:
 
 				deltatime, event = e
@@ -179,15 +179,17 @@ the format type, number of tracks and parsed time division information"""
 		# Check header
 		try:
 			if fp.read(4) != "MThd":
-				raise HeaderError, "Not a valid MIDI file header"
+				raise HeaderError, "Not a valid MIDI file header. Byte %d." % self.bytes_read
+			self.bytes_read += 4
 		except:
 			raise IOError, "Couldn't read from file."
 
 		# Parse chunk size
 		try:
 			chunk_size = self.bytes_to_int(fp.read(4))
+			self.bytes_read += 4
 		except:
-			raise IOError, "Couldn't read chunk size from file."
+			raise IOError, "Couldn't read chunk size from file. Byte %d." % self.bytes_read
 		
 		# Expect chunk size to be at least 6
 		if chunk_size < 6:
@@ -195,6 +197,7 @@ the format type, number of tracks and parsed time division information"""
 
 		try:
 			format_type = self.bytes_to_int(fp.read(2))
+			self.bytes_read += 2
 			if format_type not in [0, 1, 2]:
 				raise FormatError, "%d is not a valid MIDI format." % format_type
 		except:
@@ -203,6 +206,7 @@ the format type, number of tracks and parsed time division information"""
 		try:
 			number_of_tracks = self.bytes_to_int(fp.read(2))
 			time_division = self.parse_time_division(fp.read(2))
+			self.bytes_read += 4
 		except:
 			raise IOError, "Couldn't read number of tracks and/or time division from tracks."
 
@@ -212,6 +216,7 @@ the format type, number of tracks and parsed time division information"""
 		if chunk_size % 2 == 1:
 			raise FormatError, "Won't parse this."
 		fp.read(chunk_size / 2)
+		self.bytes_read += chunk_size / 2
 
 		return (format_type, number_of_tracks, time_division)
 
@@ -254,7 +259,10 @@ a list of events and the number of bytes that were read."""
 
 			events.append([delta_time, event])
 
-		return events, bytes
+		if chunk_size < 0:
+			print "yikes.", self.bytes_read, chunk_size
+
+		return events
 
 	def parse_midi_event(self, fp):
 		"""Parses a MIDI event. Returns a dictionary and a the number of bytes read."""
@@ -263,6 +271,7 @@ a list of events and the number of bytes that were read."""
 		try:
 			ec = self.bytes_to_int(fp.read(1))
 			chunk_size += 1
+			self.bytes_read += 1
 		except:
 			raise IOError, "Couldn't read event type and channel data from file."
 
@@ -270,8 +279,10 @@ a list of events and the number of bytes that were read."""
 		event_type = (ec & 0xf0) >> 4
 		channel = ec & 0x0f
 
+		# I don't know what these events are supposed to do, but I keep finding them.
+		# The parser ignores them.
 		if event_type < 8:
-			raise FormatError, "Unknown event type %d" % event_type
+			raise FormatError, "Unknown event type %d. Byte %d." % (event_type, self.bytes_read)
 
 		# Meta events can have strings of variable length
 		if event_type == 15:
@@ -280,49 +291,62 @@ a list of events and the number of bytes that were read."""
 				length,chunk_delta = self.parse_varbyte_as_int(fp)
 				data = fp.read(length)
 				chunk_size += 1 + chunk_delta + length
+				self.bytes_read += 1 + length
 			except:
 				raise IOError, "Couldn't read meta event from file."
+			
 			return {"event": event_type, "meta_event": meta_event,
 					"data": data}, chunk_size
 
 		# Program change and Channel aftertouch events only have one parameter
 		elif event_type in [12, 13]: 
 			try:
-				param1 = self.bytes_to_int(fp.read(1))
+				param1 = fp.read(1)
 				chunk_size += 1
+				self.bytes_read += 1
 			except:
 				raise IOError, "Couldn't read MIDI event parameters from file."
 
-			
+			param1 = self.bytes_to_int(param1)
 			return {"event": event_type, "channel": channel, 
-				"param1": param1},  chunk_size
+				"param1": param1}, chunk_size
 		else:
 			try:
-				param1 = self.bytes_to_int(fp.read(1))
-				param2 = self.bytes_to_int(fp.read(1))
+				param1 = fp.read(1)
+				param2 = fp.read(1)
 				chunk_size += 2
+				self.bytes_read += 2
 			except:
 				raise IOError, "Couldn't read MIDI event parameters from file."
-
+			param1 = self.bytes_to_int(param1)
+			param2 = self.bytes_to_int(param2)
 			
 			return {"event": event_type, "channel": channel, 
-				"param1": param1, "param2": param2},  chunk_size
+				"param1": param1, "param2": param2}, chunk_size
 
 
 
 
 	def parse_track_header(self, fp):
 		"""Returns the size of the track chunk."""
+		# Check the header
 		try:
-			if fp.read(4) != "MTrk":
-				raise HeaderError, "Not a valid Track header."""
+			h = fp.read(4)
+			self.bytes_read += 4
 		except:
-			raise IOError, "Couldn't read track header from file."""
+			raise IOError, "Couldn't read track header from file. Byte %d." % self.bytes_read
 
+		if h != "MTrk":
+			raise HeaderError, "Not a valid Track header. Byte %d." % self.bytes_read
+
+		# Parse the size of the header
 		try:
-			chunk_size = self.bytes_to_int(fp.read(4))
+			chunk_size = fp.read(4)
+			self.bytes_read += 4
 		except:
 			raise IOError, "Couldn't read track chunk size from file."""
+
+		chunk_size = self.bytes_to_int(chunk_size)
 		return chunk_size
 
 
@@ -334,19 +358,18 @@ the MIDI format, the number of tracks and the time division-, the parsed track d
 		except:
 			raise IOError, "File not found"
 		
+		self.bytes_read = 0
 		header = self.parse_midi_file_header(f)
 		tracks = header[1]
-		bytes_read = 0
 		result = []
 
 		while tracks > 0:
-			events, bytes = self.parse_track(f)
-			bytes_read += bytes
+			events = self.parse_track(f)
 			result.append(events)
 			tracks -= 1
 		
 		f.close()
-		return header, result, bytes_read
+		return (header, result)
 
 	def parse_varbyte_as_int(self, fp, return_bytes_read = True):
 		"""Reads a variable length byte from the file and returns the corresponding integer."""
@@ -356,6 +379,7 @@ the MIDI format, the number of tracks and the time division-, the parsed track d
 		while r & 0x80:
 			try:
 				r = self.bytes_to_int(fp.read(1))
+				self.bytes_read += 1
 			except:
 				IOError, "Couldn't read variable length byte from file."
 			if r & 0x80:
